# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_csm_autoscaling_defaults import ModelCSMAutoscalingDefaults


class ApimodelCSMAppLimitsResponse(Model):
    """Apimodel CSM app limits response (apimodel.CSMAppLimitsResponse)

    Properties:
        autoscaling: (autoscaling) REQUIRED ModelCSMAutoscalingDefaults

        extend_app_cpu_limit: (extendAppCPULimit) REQUIRED int

        extend_app_event_handler_cpu_limit: (extendAppEventHandlerCPULimit) REQUIRED int

        extend_app_event_handler_memory_limit: (extendAppEventHandlerMemoryLimit) REQUIRED int

        extend_app_memory_limit: (extendAppMemoryLimit) REQUIRED int

        extend_app_replica_limit: (extendAppReplicaLimit) REQUIRED int

        extend_app_service_extension_cpu_limit: (extendAppServiceExtensionCPULimit) REQUIRED int

        extend_app_service_extension_memory_limit: (extendAppServiceExtensionMemoryLimit) REQUIRED int

        extend_appe_event_handler_memory_limit: (extendAppeEventHandlerMemoryLimit) REQUIRED int

        max_subscriber_count: (maxSubscriberCount) REQUIRED int
    """

    # region fields

    autoscaling: ModelCSMAutoscalingDefaults  # REQUIRED
    extend_app_cpu_limit: int  # REQUIRED
    extend_app_event_handler_cpu_limit: int  # REQUIRED
    extend_app_event_handler_memory_limit: int  # REQUIRED
    extend_app_memory_limit: int  # REQUIRED
    extend_app_replica_limit: int  # REQUIRED
    extend_app_service_extension_cpu_limit: int  # REQUIRED
    extend_app_service_extension_memory_limit: int  # REQUIRED
    extend_appe_event_handler_memory_limit: int  # REQUIRED
    max_subscriber_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_autoscaling(
        self, value: ModelCSMAutoscalingDefaults
    ) -> ApimodelCSMAppLimitsResponse:
        self.autoscaling = value
        return self

    def with_extend_app_cpu_limit(self, value: int) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_cpu_limit = value
        return self

    def with_extend_app_event_handler_cpu_limit(
        self, value: int
    ) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_event_handler_cpu_limit = value
        return self

    def with_extend_app_event_handler_memory_limit(
        self, value: int
    ) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_event_handler_memory_limit = value
        return self

    def with_extend_app_memory_limit(self, value: int) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_memory_limit = value
        return self

    def with_extend_app_replica_limit(self, value: int) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_replica_limit = value
        return self

    def with_extend_app_service_extension_cpu_limit(
        self, value: int
    ) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_service_extension_cpu_limit = value
        return self

    def with_extend_app_service_extension_memory_limit(
        self, value: int
    ) -> ApimodelCSMAppLimitsResponse:
        self.extend_app_service_extension_memory_limit = value
        return self

    def with_extend_appe_event_handler_memory_limit(
        self, value: int
    ) -> ApimodelCSMAppLimitsResponse:
        self.extend_appe_event_handler_memory_limit = value
        return self

    def with_max_subscriber_count(self, value: int) -> ApimodelCSMAppLimitsResponse:
        self.max_subscriber_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "autoscaling"):
            result["autoscaling"] = self.autoscaling.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["autoscaling"] = ModelCSMAutoscalingDefaults()
        if hasattr(self, "extend_app_cpu_limit"):
            result["extendAppCPULimit"] = int(self.extend_app_cpu_limit)
        elif include_empty:
            result["extendAppCPULimit"] = 0
        if hasattr(self, "extend_app_event_handler_cpu_limit"):
            result["extendAppEventHandlerCPULimit"] = int(
                self.extend_app_event_handler_cpu_limit
            )
        elif include_empty:
            result["extendAppEventHandlerCPULimit"] = 0
        if hasattr(self, "extend_app_event_handler_memory_limit"):
            result["extendAppEventHandlerMemoryLimit"] = int(
                self.extend_app_event_handler_memory_limit
            )
        elif include_empty:
            result["extendAppEventHandlerMemoryLimit"] = 0
        if hasattr(self, "extend_app_memory_limit"):
            result["extendAppMemoryLimit"] = int(self.extend_app_memory_limit)
        elif include_empty:
            result["extendAppMemoryLimit"] = 0
        if hasattr(self, "extend_app_replica_limit"):
            result["extendAppReplicaLimit"] = int(self.extend_app_replica_limit)
        elif include_empty:
            result["extendAppReplicaLimit"] = 0
        if hasattr(self, "extend_app_service_extension_cpu_limit"):
            result["extendAppServiceExtensionCPULimit"] = int(
                self.extend_app_service_extension_cpu_limit
            )
        elif include_empty:
            result["extendAppServiceExtensionCPULimit"] = 0
        if hasattr(self, "extend_app_service_extension_memory_limit"):
            result["extendAppServiceExtensionMemoryLimit"] = int(
                self.extend_app_service_extension_memory_limit
            )
        elif include_empty:
            result["extendAppServiceExtensionMemoryLimit"] = 0
        if hasattr(self, "extend_appe_event_handler_memory_limit"):
            result["extendAppeEventHandlerMemoryLimit"] = int(
                self.extend_appe_event_handler_memory_limit
            )
        elif include_empty:
            result["extendAppeEventHandlerMemoryLimit"] = 0
        if hasattr(self, "max_subscriber_count"):
            result["maxSubscriberCount"] = int(self.max_subscriber_count)
        elif include_empty:
            result["maxSubscriberCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        autoscaling: ModelCSMAutoscalingDefaults,
        extend_app_cpu_limit: int,
        extend_app_event_handler_cpu_limit: int,
        extend_app_event_handler_memory_limit: int,
        extend_app_memory_limit: int,
        extend_app_replica_limit: int,
        extend_app_service_extension_cpu_limit: int,
        extend_app_service_extension_memory_limit: int,
        extend_appe_event_handler_memory_limit: int,
        max_subscriber_count: int,
        **kwargs,
    ) -> ApimodelCSMAppLimitsResponse:
        instance = cls()
        instance.autoscaling = autoscaling
        instance.extend_app_cpu_limit = extend_app_cpu_limit
        instance.extend_app_event_handler_cpu_limit = extend_app_event_handler_cpu_limit
        instance.extend_app_event_handler_memory_limit = (
            extend_app_event_handler_memory_limit
        )
        instance.extend_app_memory_limit = extend_app_memory_limit
        instance.extend_app_replica_limit = extend_app_replica_limit
        instance.extend_app_service_extension_cpu_limit = (
            extend_app_service_extension_cpu_limit
        )
        instance.extend_app_service_extension_memory_limit = (
            extend_app_service_extension_memory_limit
        )
        instance.extend_appe_event_handler_memory_limit = (
            extend_appe_event_handler_memory_limit
        )
        instance.max_subscriber_count = max_subscriber_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelCSMAppLimitsResponse:
        instance = cls()
        if not dict_:
            return instance
        if "autoscaling" in dict_ and dict_["autoscaling"] is not None:
            instance.autoscaling = ModelCSMAutoscalingDefaults.create_from_dict(
                dict_["autoscaling"], include_empty=include_empty
            )
        elif include_empty:
            instance.autoscaling = ModelCSMAutoscalingDefaults()
        if "extendAppCPULimit" in dict_ and dict_["extendAppCPULimit"] is not None:
            instance.extend_app_cpu_limit = int(dict_["extendAppCPULimit"])
        elif include_empty:
            instance.extend_app_cpu_limit = 0
        if (
            "extendAppEventHandlerCPULimit" in dict_
            and dict_["extendAppEventHandlerCPULimit"] is not None
        ):
            instance.extend_app_event_handler_cpu_limit = int(
                dict_["extendAppEventHandlerCPULimit"]
            )
        elif include_empty:
            instance.extend_app_event_handler_cpu_limit = 0
        if (
            "extendAppEventHandlerMemoryLimit" in dict_
            and dict_["extendAppEventHandlerMemoryLimit"] is not None
        ):
            instance.extend_app_event_handler_memory_limit = int(
                dict_["extendAppEventHandlerMemoryLimit"]
            )
        elif include_empty:
            instance.extend_app_event_handler_memory_limit = 0
        if (
            "extendAppMemoryLimit" in dict_
            and dict_["extendAppMemoryLimit"] is not None
        ):
            instance.extend_app_memory_limit = int(dict_["extendAppMemoryLimit"])
        elif include_empty:
            instance.extend_app_memory_limit = 0
        if (
            "extendAppReplicaLimit" in dict_
            and dict_["extendAppReplicaLimit"] is not None
        ):
            instance.extend_app_replica_limit = int(dict_["extendAppReplicaLimit"])
        elif include_empty:
            instance.extend_app_replica_limit = 0
        if (
            "extendAppServiceExtensionCPULimit" in dict_
            and dict_["extendAppServiceExtensionCPULimit"] is not None
        ):
            instance.extend_app_service_extension_cpu_limit = int(
                dict_["extendAppServiceExtensionCPULimit"]
            )
        elif include_empty:
            instance.extend_app_service_extension_cpu_limit = 0
        if (
            "extendAppServiceExtensionMemoryLimit" in dict_
            and dict_["extendAppServiceExtensionMemoryLimit"] is not None
        ):
            instance.extend_app_service_extension_memory_limit = int(
                dict_["extendAppServiceExtensionMemoryLimit"]
            )
        elif include_empty:
            instance.extend_app_service_extension_memory_limit = 0
        if (
            "extendAppeEventHandlerMemoryLimit" in dict_
            and dict_["extendAppeEventHandlerMemoryLimit"] is not None
        ):
            instance.extend_appe_event_handler_memory_limit = int(
                dict_["extendAppeEventHandlerMemoryLimit"]
            )
        elif include_empty:
            instance.extend_appe_event_handler_memory_limit = 0
        if "maxSubscriberCount" in dict_ and dict_["maxSubscriberCount"] is not None:
            instance.max_subscriber_count = int(dict_["maxSubscriberCount"])
        elif include_empty:
            instance.max_subscriber_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelCSMAppLimitsResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelCSMAppLimitsResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelCSMAppLimitsResponse,
        List[ApimodelCSMAppLimitsResponse],
        Dict[Any, ApimodelCSMAppLimitsResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "autoscaling": "autoscaling",
            "extendAppCPULimit": "extend_app_cpu_limit",
            "extendAppEventHandlerCPULimit": "extend_app_event_handler_cpu_limit",
            "extendAppEventHandlerMemoryLimit": "extend_app_event_handler_memory_limit",
            "extendAppMemoryLimit": "extend_app_memory_limit",
            "extendAppReplicaLimit": "extend_app_replica_limit",
            "extendAppServiceExtensionCPULimit": "extend_app_service_extension_cpu_limit",
            "extendAppServiceExtensionMemoryLimit": "extend_app_service_extension_memory_limit",
            "extendAppeEventHandlerMemoryLimit": "extend_appe_event_handler_memory_limit",
            "maxSubscriberCount": "max_subscriber_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "autoscaling": True,
            "extendAppCPULimit": True,
            "extendAppEventHandlerCPULimit": True,
            "extendAppEventHandlerMemoryLimit": True,
            "extendAppMemoryLimit": True,
            "extendAppReplicaLimit": True,
            "extendAppServiceExtensionCPULimit": True,
            "extendAppServiceExtensionMemoryLimit": True,
            "extendAppeEventHandlerMemoryLimit": True,
            "maxSubscriberCount": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ErrorEntity
from ..models import FulfillmentScriptCreate
from ..models import FulfillmentScriptInfo
from ..models import FulfillmentScriptUpdate

from ..operations.fulfillment_script import CreateFulfillmentScript
from ..operations.fulfillment_script import DeleteFulfillmentScript
from ..operations.fulfillment_script import GetFulfillmentScript
from ..operations.fulfillment_script import ListFulfillmentScripts
from ..operations.fulfillment_script import UpdateFulfillmentScript


@same_doc_as(CreateFulfillmentScript)
def create_fulfillment_script(
    id_: str,
    body: Optional[FulfillmentScriptCreate] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create fulfillment script (createFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Create fulfillment script.
    Other detail info:


    Fulfillment scripts are used for adding custom fulfillment logic based on ITEM_TYPE : [MEDIA,INGAMEITEM] for now, and the custom scripts only cover grantDays.
    Example for grantDays:
    `order && ((order.currency && order.currency.currencyCode) == 'LP' || order.isFree) ? 30 : -1`

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: POST

        tags: ["FulfillmentScript"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL FulfillmentScriptCreate in body

        id_: (id) REQUIRED str in path

    Responses:
        201: Created - FulfillmentScriptInfo (successful operation)

        409: Conflict - ErrorEntity (38171: Fulfillment script already exists)
    """
    request = CreateFulfillmentScript.create(
        id_=id_,
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateFulfillmentScript)
async def create_fulfillment_script_async(
    id_: str,
    body: Optional[FulfillmentScriptCreate] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create fulfillment script (createFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Create fulfillment script.
    Other detail info:


    Fulfillment scripts are used for adding custom fulfillment logic based on ITEM_TYPE : [MEDIA,INGAMEITEM] for now, and the custom scripts only cover grantDays.
    Example for grantDays:
    `order && ((order.currency && order.currency.currencyCode) == 'LP' || order.isFree) ? 30 : -1`

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: POST

        tags: ["FulfillmentScript"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL FulfillmentScriptCreate in body

        id_: (id) REQUIRED str in path

    Responses:
        201: Created - FulfillmentScriptInfo (successful operation)

        409: Conflict - ErrorEntity (38171: Fulfillment script already exists)
    """
    request = CreateFulfillmentScript.create(
        id_=id_,
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteFulfillmentScript)
def delete_fulfillment_script(
    id_: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Delete fulfillment script (deleteFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Delete fulfillment script.

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: DELETE

        tags: ["FulfillmentScript"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

    Responses:
        204: No Content - (delete fulfillment script successfully)
    """
    request = DeleteFulfillmentScript.create(
        id_=id_,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteFulfillmentScript)
async def delete_fulfillment_script_async(
    id_: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Delete fulfillment script (deleteFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Delete fulfillment script.

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: DELETE

        tags: ["FulfillmentScript"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

    Responses:
        204: No Content - (delete fulfillment script successfully)
    """
    request = DeleteFulfillmentScript.create(
        id_=id_,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetFulfillmentScript)
def get_fulfillment_script(
    id_: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get fulfillment script by id (getFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Get fulfillment script by id.
    Other detail info:

      * Returns : get fulfillment script

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: GET

        tags: ["FulfillmentScript"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

    Responses:
        200: OK - FulfillmentScriptInfo (successful operation)

        404: Not Found - ErrorEntity (38141: Fulfillment script does not exist)
    """
    request = GetFulfillmentScript.create(
        id_=id_,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetFulfillmentScript)
async def get_fulfillment_script_async(
    id_: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Get fulfillment script by id (getFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Get fulfillment script by id.
    Other detail info:

      * Returns : get fulfillment script

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: GET

        tags: ["FulfillmentScript"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        id_: (id) REQUIRED str in path

    Responses:
        200: OK - FulfillmentScriptInfo (successful operation)

        404: Not Found - ErrorEntity (38141: Fulfillment script does not exist)
    """
    request = GetFulfillmentScript.create(
        id_=id_,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListFulfillmentScripts)
def list_fulfillment_scripts(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """List all fulfillment scripts (listFulfillmentScripts)

    [Not supported yet in AGS Shared Cloud] List all fulfillment scripts.

    Properties:
        url: /platform/admin/fulfillment/scripts

        method: GET

        tags: ["FulfillmentScript"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[FulfillmentScriptInfo] (successful operation)
    """
    request = ListFulfillmentScripts.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListFulfillmentScripts)
async def list_fulfillment_scripts_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """List all fulfillment scripts (listFulfillmentScripts)

    [Not supported yet in AGS Shared Cloud] List all fulfillment scripts.

    Properties:
        url: /platform/admin/fulfillment/scripts

        method: GET

        tags: ["FulfillmentScript"]

        consumes: []

        produces: []

        securities: [BEARER_AUTH]

    Responses:
        200: OK - List[FulfillmentScriptInfo] (successful operation)
    """
    request = ListFulfillmentScripts.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UpdateFulfillmentScript)
def update_fulfillment_script(
    id_: str,
    body: Optional[FulfillmentScriptUpdate] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update fulfillment script (updateFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Update fulfillment script.

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: PATCH

        tags: ["FulfillmentScript"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL FulfillmentScriptUpdate in body

        id_: (id) REQUIRED str in path

    Responses:
        200: OK - FulfillmentScriptInfo (successful operation)

        400: Bad Request - ErrorEntity (38141: Fulfillment script does not exist)
    """
    request = UpdateFulfillmentScript.create(
        id_=id_,
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UpdateFulfillmentScript)
async def update_fulfillment_script_async(
    id_: str,
    body: Optional[FulfillmentScriptUpdate] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update fulfillment script (updateFulfillmentScript)

    [Not supported yet in AGS Shared Cloud] Update fulfillment script.

    Properties:
        url: /platform/admin/fulfillment/scripts/{id}

        method: PATCH

        tags: ["FulfillmentScript"]

        consumes: ["application/json"]

        produces: []

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL FulfillmentScriptUpdate in body

        id_: (id) REQUIRED str in path

    Responses:
        200: OK - FulfillmentScriptInfo (successful operation)

        400: Bad Request - ErrorEntity (38141: Fulfillment script does not exist)
    """
    request = UpdateFulfillmentScript.create(
        id_=id_,
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import deprecated
from ....core import same_doc_as

from ..models import CreateBasePolicyRequest
from ..models import CreateBasePolicyResponse
from ..models import ErrorEntity
from ..models import RetrieveBasePolicyResponse
from ..models import RetrievePolicyResponse
from ..models import RetrievePolicyTypeResponse
from ..models import UpdateBasePolicyRequest
from ..models import UpdateBasePolicyResponse
from ..models import ValidationErrorEntity

from ..operations.base_legal_policies import OldCreatePolicy
from ..operations.base_legal_policies import OldPartialUpdatePolicy
from ..operations.base_legal_policies import OldRetrieveAllPolicyTypes
from ..operations.base_legal_policies import OldRetrievePolicyCountry
from ..operations.base_legal_policies import OldRetrieveSinglePolicy
from ..operations.base_legal_policies import RetrieveAllLegalPolicies
from ..models import CreateBasePolicyRequestCountryTypeEnum
from ..models import CreateBasePolicyResponseCountryTypeEnum
from ..models import UpdateBasePolicyRequestCountryTypeEnum


@deprecated
@same_doc_as(OldCreatePolicy)
def old_create_policy(
    body: Optional[CreateBasePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Base Legal Policy (oldCreatePolicy)

    Create a legal policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/base-policies

        method: POST

        tags: ["Base Legal Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateBasePolicyRequest in body

    Responses:
        201: Created - CreateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40038: errors.net.accelbyte.platform.legal.invalid_affected_client_id | 40026: errors.net.accelbyte.platform.legal.not_allow_create_studio_policy)

        404: Not Found - ErrorEntity (40030: errors.net.accelbyte.platform.legal.policy_type_not_exist)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    request = OldCreatePolicy.create(
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(OldCreatePolicy)
async def old_create_policy_async(
    body: Optional[CreateBasePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a Base Legal Policy (oldCreatePolicy)

    Create a legal policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/base-policies

        method: POST

        tags: ["Base Legal Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL CreateBasePolicyRequest in body

    Responses:
        201: Created - CreateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40038: errors.net.accelbyte.platform.legal.invalid_affected_client_id | 40026: errors.net.accelbyte.platform.legal.not_allow_create_studio_policy)

        404: Not Found - ErrorEntity (40030: errors.net.accelbyte.platform.legal.policy_type_not_exist)

        422: Unprocessable Entity - ValidationErrorEntity (20002: validation error)
    """
    request = OldCreatePolicy.create(
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(OldPartialUpdatePolicy)
def old_partial_update_policy(
    base_policy_id: str,
    body: Optional[UpdateBasePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Base Legal Policy (oldPartialUpdatePolicy)

    Update an existing base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}

        method: PATCH

        tags: ["Base Legal Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateBasePolicyRequest in body

        base_policy_id: (basePolicyId) REQUIRED str in path

    Responses:
        200: OK - UpdateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy | 40038: errors.net.accelbyte.platform.legal.invalid_affected_client_id)

        404: Not Found - ErrorEntity (40030: errors.net.accelbyte.platform.legal.policy_type_not_exist)
    """
    request = OldPartialUpdatePolicy.create(
        base_policy_id=base_policy_id,
        body=body,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(OldPartialUpdatePolicy)
async def old_partial_update_policy_async(
    base_policy_id: str,
    body: Optional[UpdateBasePolicyRequest] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Update Base Legal Policy (oldPartialUpdatePolicy)

    Update an existing base policy.
    Note:


      *  countryType field only accept COUNTRY or COUNTRY_GROUP.
      * COUNTRY_GROUP countryType requires the countryGroupName and affectedCountries fields to be provided.


    * policy with COUNTRY_GROUP type include multiple countries and apply the same policy across the entire list of countries

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}

        method: PATCH

        tags: ["Base Legal Policies"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) OPTIONAL UpdateBasePolicyRequest in body

        base_policy_id: (basePolicyId) REQUIRED str in path

    Responses:
        200: OK - UpdateBasePolicyResponse (successful operation)

        400: Bad Request - ErrorEntity (40032: errors.net.accelbyte.platform.legal.invalid_base_policy | 40038: errors.net.accelbyte.platform.legal.invalid_affected_client_id)

        404: Not Found - ErrorEntity (40030: errors.net.accelbyte.platform.legal.policy_type_not_exist)
    """
    request = OldPartialUpdatePolicy.create(
        base_policy_id=base_policy_id,
        body=body,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(OldRetrieveAllPolicyTypes)
def old_retrieve_all_policy_types(
    limit: int,
    offset: Optional[int] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Policy Type (oldRetrieveAllPolicyTypes)

    Retrieve all supported policy types.

    Properties:
        url: /agreement/admin/policy-types

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        offset: (offset) OPTIONAL int in query

        limit: (limit) REQUIRED int in query

    Responses:
        200: OK - List[RetrievePolicyTypeResponse] (successful operation)
    """
    request = OldRetrieveAllPolicyTypes.create(
        limit=limit,
        offset=offset,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(OldRetrieveAllPolicyTypes)
async def old_retrieve_all_policy_types_async(
    limit: int,
    offset: Optional[int] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Policy Type (oldRetrieveAllPolicyTypes)

    Retrieve all supported policy types.

    Properties:
        url: /agreement/admin/policy-types

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        offset: (offset) OPTIONAL int in query

        limit: (limit) REQUIRED int in query

    Responses:
        200: OK - List[RetrievePolicyTypeResponse] (successful operation)
    """
    request = OldRetrieveAllPolicyTypes.create(
        limit=limit,
        offset=offset,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(OldRetrievePolicyCountry)
def old_retrieve_policy_country(
    base_policy_id: str,
    country_code: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy based on a Particular Country (oldRetrievePolicyCountry)

    Retrieve a Base Legal Policy based on a Particular Country.

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}/countries/{countryCode}

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        country_code: (countryCode) REQUIRED str in path

    Responses:
        200: OK - RetrievePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    request = OldRetrievePolicyCountry.create(
        base_policy_id=base_policy_id,
        country_code=country_code,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(OldRetrievePolicyCountry)
async def old_retrieve_policy_country_async(
    base_policy_id: str,
    country_code: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve a Base Legal Policy based on a Particular Country (oldRetrievePolicyCountry)

    Retrieve a Base Legal Policy based on a Particular Country.

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}/countries/{countryCode}

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

        country_code: (countryCode) REQUIRED str in path

    Responses:
        200: OK - RetrievePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40041: errors.net.accelbyte.platform.legal.policy_not_found)
    """
    request = OldRetrievePolicyCountry.create(
        base_policy_id=base_policy_id,
        country_code=country_code,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(OldRetrieveSinglePolicy)
def old_retrieve_single_policy(
    base_policy_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve a Base Legal Policy (oldRetrieveSinglePolicy)

    Retrieve a base policy.

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

    Responses:
        200: OK - RetrieveBasePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40031: errors.net.accelbyte.platform.legal.base_policy_not_found)
    """
    request = OldRetrieveSinglePolicy.create(
        base_policy_id=base_policy_id,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(OldRetrieveSinglePolicy)
async def old_retrieve_single_policy_async(
    base_policy_id: str, x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve a Base Legal Policy (oldRetrieveSinglePolicy)

    Retrieve a base policy.

    Properties:
        url: /agreement/admin/base-policies/{basePolicyId}

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        base_policy_id: (basePolicyId) REQUIRED str in path

    Responses:
        200: OK - RetrieveBasePolicyResponse (successful operation)

        404: Not Found - ErrorEntity (40031: errors.net.accelbyte.platform.legal.base_policy_not_found)
    """
    request = OldRetrieveSinglePolicy.create(
        base_policy_id=base_policy_id,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@deprecated
@same_doc_as(RetrieveAllLegalPolicies)
def retrieve_all_legal_policies(
    visible_only: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Base Legal Policy (retrieveAllLegalPolicies)

    Retrieve all base policies.

    Properties:
        url: /agreement/admin/base-policies

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrieveBasePolicyResponse] (successful operation)
    """
    request = RetrieveAllLegalPolicies.create(
        visible_only=visible_only,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@deprecated
@same_doc_as(RetrieveAllLegalPolicies)
async def retrieve_all_legal_policies_async(
    visible_only: Optional[bool] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Retrieve All Base Legal Policy (retrieveAllLegalPolicies)

    Retrieve all base policies.

    Properties:
        url: /agreement/admin/base-policies

        method: GET

        tags: ["Base Legal Policies"]

        consumes: []

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        visible_only: (visibleOnly) OPTIONAL bool in query

    Responses:
        200: OK - List[RetrieveBasePolicyResponse] (successful operation)
    """
    request = RetrieveAllLegalPolicies.create(
        visible_only=visible_only,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Login Queue Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model


class ApimodelsQueueStatusResponse(Model):
    """Apimodels queue status response (apimodels.QueueStatusResponse)

    Properties:
        is_active: (isActive) REQUIRED bool

        logged_user_count: (loggedUserCount) REQUIRED int

        login_rps: (loginRPS) REQUIRED int

        namespace: (namespace) REQUIRED str

        queue_count: (queueCount) REQUIRED int
    """

    # region fields

    is_active: bool  # REQUIRED
    logged_user_count: int  # REQUIRED
    login_rps: int  # REQUIRED
    namespace: str  # REQUIRED
    queue_count: int  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_is_active(self, value: bool) -> ApimodelsQueueStatusResponse:
        self.is_active = value
        return self

    def with_logged_user_count(self, value: int) -> ApimodelsQueueStatusResponse:
        self.logged_user_count = value
        return self

    def with_login_rps(self, value: int) -> ApimodelsQueueStatusResponse:
        self.login_rps = value
        return self

    def with_namespace(self, value: str) -> ApimodelsQueueStatusResponse:
        self.namespace = value
        return self

    def with_queue_count(self, value: int) -> ApimodelsQueueStatusResponse:
        self.queue_count = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "is_active"):
            result["isActive"] = bool(self.is_active)
        elif include_empty:
            result["isActive"] = False
        if hasattr(self, "logged_user_count"):
            result["loggedUserCount"] = int(self.logged_user_count)
        elif include_empty:
            result["loggedUserCount"] = 0
        if hasattr(self, "login_rps"):
            result["loginRPS"] = int(self.login_rps)
        elif include_empty:
            result["loginRPS"] = 0
        if hasattr(self, "namespace"):
            result["namespace"] = str(self.namespace)
        elif include_empty:
            result["namespace"] = ""
        if hasattr(self, "queue_count"):
            result["queueCount"] = int(self.queue_count)
        elif include_empty:
            result["queueCount"] = 0
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        is_active: bool,
        logged_user_count: int,
        login_rps: int,
        namespace: str,
        queue_count: int,
        **kwargs,
    ) -> ApimodelsQueueStatusResponse:
        instance = cls()
        instance.is_active = is_active
        instance.logged_user_count = logged_user_count
        instance.login_rps = login_rps
        instance.namespace = namespace
        instance.queue_count = queue_count
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelsQueueStatusResponse:
        instance = cls()
        if not dict_:
            return instance
        if "isActive" in dict_ and dict_["isActive"] is not None:
            instance.is_active = bool(dict_["isActive"])
        elif include_empty:
            instance.is_active = False
        if "loggedUserCount" in dict_ and dict_["loggedUserCount"] is not None:
            instance.logged_user_count = int(dict_["loggedUserCount"])
        elif include_empty:
            instance.logged_user_count = 0
        if "loginRPS" in dict_ and dict_["loginRPS"] is not None:
            instance.login_rps = int(dict_["loginRPS"])
        elif include_empty:
            instance.login_rps = 0
        if "namespace" in dict_ and dict_["namespace"] is not None:
            instance.namespace = str(dict_["namespace"])
        elif include_empty:
            instance.namespace = ""
        if "queueCount" in dict_ and dict_["queueCount"] is not None:
            instance.queue_count = int(dict_["queueCount"])
        elif include_empty:
            instance.queue_count = 0
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelsQueueStatusResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelsQueueStatusResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelsQueueStatusResponse,
        List[ApimodelsQueueStatusResponse],
        Dict[Any, ApimodelsQueueStatusResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "isActive": "is_active",
            "loggedUserCount": "logged_user_count",
            "loginRPS": "login_rps",
            "namespace": "namespace",
            "queueCount": "queue_count",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "isActive": True,
            "loggedUserCount": True,
            "loginRPS": True,
            "namespace": True,
            "queueCount": True,
        }

    # endregion static methods



# Copyright (c) 2025 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.

from logging import Logger, getLogger

from ...core import NoHttpBackoffPolicy, NoHttpRetryPolicy

MODULE_NAME = "accelbyte_py_sdk"

LOGIN_TYPE_CLIENT: str = "client"
LOGIN_TYPE_PLATFORM: str = "platform"
LOGIN_TYPE_USER: str = "user"
LOGIN_TYPE_REFRESHABLE: str = "refreshable"

DEFAULT_MAX_RETRY: int = 2
DEFAULT_REFRESH_RATE: float = 0.8

DEFAULT_SCOPE: str = "commerce account social publishing analytics"

DEFAULT_AUTO_REFRESH: bool = False

DEFAULT_TIMER_INTERVAL: int = 60
DEFAULT_TIMER_REFRESH_RATE: float = DEFAULT_REFRESH_RATE
DEFAULT_TIMER_REPEATS: int = -1
DEFAULT_TIMER_REPEAT_ON_EXCEPTION: bool = True
DEFAULT_TIMER_AUTOSTART: bool = False

AUTH_SERVICE_LOGGER: Logger = getLogger(f"{MODULE_NAME}.services.auth")

REFRESH_KWARGS = {
    "retry_policy": NoHttpRetryPolicy(),
    "backoff_policy": NoHttpBackoffPolicy(),
}



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApimodelAppUIResponse
from ..models import ApimodelCreateAppUIRequest
from ..models import ApimodelListAppUIResponse
from ..models import ApimodelUploadFileResponse
from ..models import ResponseErrorResponse

from ..operations.app_ui import CreateAppUI
from ..operations.app_ui import DeleteAppUI
from ..operations.app_ui import ListAppUI
from ..operations.app_ui import UploadAppUIFile


@same_doc_as(CreateAppUI)
def create_app_ui(
    body: ApimodelCreateAppUIRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a new App UI instance (CreateAppUI)

    Creates a new App UI configuration. The App UI can be hosted either by AccelByte (default) or externally.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui

        method: POST

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateAppUIRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelAppUIResponse (Created)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateAppUI.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateAppUI)
async def create_app_ui_async(
    body: ApimodelCreateAppUIRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a new App UI instance (CreateAppUI)

    Creates a new App UI configuration. The App UI can be hosted either by AccelByte (default) or externally.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui

        method: POST

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApimodelCreateAppUIRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApimodelAppUIResponse (Created)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        409: Conflict - ResponseErrorResponse (Conflict)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateAppUI.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteAppUI)
def delete_app_ui(
    app_ui_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an App UI instance (DeleteAppUI)

    Permanently deletes an App UI instance and schedules cleanup of associated S3 assets.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui/{appUiName}

        method: DELETE

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app_ui_name: (appUiName) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteAppUI.create(
        app_ui_name=app_ui_name,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteAppUI)
async def delete_app_ui_async(
    app_ui_name: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete an App UI instance (DeleteAppUI)

    Permanently deletes an App UI instance and schedules cleanup of associated S3 assets.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui/{appUiName}

        method: DELETE

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        app_ui_name: (appUiName) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (No Content)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteAppUI.create(
        app_ui_name=app_ui_name,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(ListAppUI)
def list_app_ui(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all App UI instances in the namespace (ListAppUI)

    Retrieves a paginated list of App UI instances. Supports filtering by name and pagination using limit/offset.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui

        method: GET

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelListAppUIResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ListAppUI.create(
        limit=limit,
        name=name,
        offset=offset,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(ListAppUI)
async def list_app_ui_async(
    limit: Optional[int] = None,
    name: Optional[str] = None,
    offset: Optional[int] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """List all App UI instances in the namespace (ListAppUI)

    Retrieves a paginated list of App UI instances. Supports filtering by name and pagination using limit/offset.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui

        method: GET

        tags: ["App UI"]

        consumes: ["application/json", "multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        limit: (limit) OPTIONAL int in query

        name: (name) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

    Responses:
        200: OK - ApimodelListAppUIResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = ListAppUI.create(
        limit=limit,
        name=name,
        offset=offset,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(UploadAppUIFile)
def upload_app_ui_file(
    app_ui_name: str,
    file: Any,
    version: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Upload App UI assets as a zip file (UploadAppUIFile)

    Uploads a ZIP archive containing App UI static assets (HTML, CSS, JS, etc.). Files are stored in S3 and served with immutable caching.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui/{appUiName}/files/upload

        method: POST

        tags: ["App UI"]

        consumes: ["multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        file: (file) REQUIRED Any in form_data

        app_ui_name: (appUiName) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) OPTIONAL str in query

    Responses:
        200: OK - ApimodelUploadFileResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)

        502: Bad Gateway - ResponseErrorResponse (Bad Gateway)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UploadAppUIFile.create(
        app_ui_name=app_ui_name,
        file=file,
        version=version,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(UploadAppUIFile)
async def upload_app_ui_file_async(
    app_ui_name: str,
    file: Any,
    version: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Upload App UI assets as a zip file (UploadAppUIFile)

    Uploads a ZIP archive containing App UI static assets (HTML, CSS, JS, etc.). Files are stored in S3 and served with immutable caching.

    Properties:
        url: /csm/v1/admin/namespaces/{namespace}/app-ui/{appUiName}/files/upload

        method: POST

        tags: ["App UI"]

        consumes: ["multipart/form-data"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        file: (file) REQUIRED Any in form_data

        app_ui_name: (appUiName) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

        version: (version) OPTIONAL str in query

    Responses:
        200: OK - ApimodelUploadFileResponse (OK)

        400: Bad Request - ResponseErrorResponse (Bad Request)

        404: Not Found - ResponseErrorResponse (Not Found)

        500: Internal Server Error - ResponseErrorResponse (Internal Server Error)

        502: Bad Gateway - ResponseErrorResponse (Bad Gateway)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = UploadAppUIFile.create(
        app_ui_name=app_ui_name,
        file=file,
        version=version,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.apimodel_public_iam_client_response import ApimodelPublicIAMClientResponse


class ApimodelAppUIResponse(Model):
    """Apimodel app UI response (apimodel.AppUIResponse)

    Properties:
        app_ui_id: (appUiId) REQUIRED str

        asset_base_path: (assetBasePath) REQUIRED str

        created_at: (createdAt) REQUIRED str

        current_file_version: (currentFileVersion) REQUIRED str

        game_namespace: (gameNamespace) REQUIRED str

        name: (name) REQUIRED str

        updated_at: (updatedAt) REQUIRED str

        public_iam_client: (publicIamClient) OPTIONAL ApimodelPublicIAMClientResponse
    """

    # region fields

    app_ui_id: str  # REQUIRED
    asset_base_path: str  # REQUIRED
    created_at: str  # REQUIRED
    current_file_version: str  # REQUIRED
    game_namespace: str  # REQUIRED
    name: str  # REQUIRED
    updated_at: str  # REQUIRED
    public_iam_client: ApimodelPublicIAMClientResponse  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_app_ui_id(self, value: str) -> ApimodelAppUIResponse:
        self.app_ui_id = value
        return self

    def with_asset_base_path(self, value: str) -> ApimodelAppUIResponse:
        self.asset_base_path = value
        return self

    def with_created_at(self, value: str) -> ApimodelAppUIResponse:
        self.created_at = value
        return self

    def with_current_file_version(self, value: str) -> ApimodelAppUIResponse:
        self.current_file_version = value
        return self

    def with_game_namespace(self, value: str) -> ApimodelAppUIResponse:
        self.game_namespace = value
        return self

    def with_name(self, value: str) -> ApimodelAppUIResponse:
        self.name = value
        return self

    def with_updated_at(self, value: str) -> ApimodelAppUIResponse:
        self.updated_at = value
        return self

    def with_public_iam_client(
        self, value: ApimodelPublicIAMClientResponse
    ) -> ApimodelAppUIResponse:
        self.public_iam_client = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "app_ui_id"):
            result["appUiId"] = str(self.app_ui_id)
        elif include_empty:
            result["appUiId"] = ""
        if hasattr(self, "asset_base_path"):
            result["assetBasePath"] = str(self.asset_base_path)
        elif include_empty:
            result["assetBasePath"] = ""
        if hasattr(self, "created_at"):
            result["createdAt"] = str(self.created_at)
        elif include_empty:
            result["createdAt"] = ""
        if hasattr(self, "current_file_version"):
            result["currentFileVersion"] = str(self.current_file_version)
        elif include_empty:
            result["currentFileVersion"] = ""
        if hasattr(self, "game_namespace"):
            result["gameNamespace"] = str(self.game_namespace)
        elif include_empty:
            result["gameNamespace"] = ""
        if hasattr(self, "name"):
            result["name"] = str(self.name)
        elif include_empty:
            result["name"] = ""
        if hasattr(self, "updated_at"):
            result["updatedAt"] = str(self.updated_at)
        elif include_empty:
            result["updatedAt"] = ""
        if hasattr(self, "public_iam_client"):
            result["publicIamClient"] = self.public_iam_client.to_dict(
                include_empty=include_empty
            )
        elif include_empty:
            result["publicIamClient"] = ApimodelPublicIAMClientResponse()
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        app_ui_id: str,
        asset_base_path: str,
        created_at: str,
        current_file_version: str,
        game_namespace: str,
        name: str,
        updated_at: str,
        public_iam_client: Optional[ApimodelPublicIAMClientResponse] = None,
        **kwargs,
    ) -> ApimodelAppUIResponse:
        instance = cls()
        instance.app_ui_id = app_ui_id
        instance.asset_base_path = asset_base_path
        instance.created_at = created_at
        instance.current_file_version = current_file_version
        instance.game_namespace = game_namespace
        instance.name = name
        instance.updated_at = updated_at
        if public_iam_client is not None:
            instance.public_iam_client = public_iam_client
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelAppUIResponse:
        instance = cls()
        if not dict_:
            return instance
        if "appUiId" in dict_ and dict_["appUiId"] is not None:
            instance.app_ui_id = str(dict_["appUiId"])
        elif include_empty:
            instance.app_ui_id = ""
        if "assetBasePath" in dict_ and dict_["assetBasePath"] is not None:
            instance.asset_base_path = str(dict_["assetBasePath"])
        elif include_empty:
            instance.asset_base_path = ""
        if "createdAt" in dict_ and dict_["createdAt"] is not None:
            instance.created_at = str(dict_["createdAt"])
        elif include_empty:
            instance.created_at = ""
        if "currentFileVersion" in dict_ and dict_["currentFileVersion"] is not None:
            instance.current_file_version = str(dict_["currentFileVersion"])
        elif include_empty:
            instance.current_file_version = ""
        if "gameNamespace" in dict_ and dict_["gameNamespace"] is not None:
            instance.game_namespace = str(dict_["gameNamespace"])
        elif include_empty:
            instance.game_namespace = ""
        if "name" in dict_ and dict_["name"] is not None:
            instance.name = str(dict_["name"])
        elif include_empty:
            instance.name = ""
        if "updatedAt" in dict_ and dict_["updatedAt"] is not None:
            instance.updated_at = str(dict_["updatedAt"])
        elif include_empty:
            instance.updated_at = ""
        if "publicIamClient" in dict_ and dict_["publicIamClient"] is not None:
            instance.public_iam_client = (
                ApimodelPublicIAMClientResponse.create_from_dict(
                    dict_["publicIamClient"], include_empty=include_empty
                )
            )
        elif include_empty:
            instance.public_iam_client = ApimodelPublicIAMClientResponse()
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelAppUIResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelAppUIResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelAppUIResponse,
        List[ApimodelAppUIResponse],
        Dict[Any, ApimodelAppUIResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "appUiId": "app_ui_id",
            "assetBasePath": "asset_base_path",
            "createdAt": "created_at",
            "currentFileVersion": "current_file_version",
            "gameNamespace": "game_namespace",
            "name": "name",
            "updatedAt": "updated_at",
            "publicIamClient": "public_iam_client",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "appUiId": True,
            "assetBasePath": True,
            "createdAt": True,
            "currentFileVersion": True,
            "gameNamespace": True,
            "name": True,
            "updatedAt": True,
            "publicIamClient": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.model_progression_info import ModelProgressionInfo


class ModelAccountProgressionInfo(Model):
    """Model account progression info (model.AccountProgressionInfo)

    Properties:
        linked_games: (linkedGames) REQUIRED List[str]

        display_name: (displayName) OPTIONAL str

        email: (email) OPTIONAL str

        platform_id: (platformId) OPTIONAL str

        progressions: (progressions) OPTIONAL List[ModelProgressionInfo]

        unique_display_name: (uniqueDisplayName) OPTIONAL str

        user_name: (userName) OPTIONAL str
    """

    # region fields

    linked_games: List[str]  # REQUIRED
    display_name: str  # OPTIONAL
    email: str  # OPTIONAL
    platform_id: str  # OPTIONAL
    progressions: List[ModelProgressionInfo]  # OPTIONAL
    unique_display_name: str  # OPTIONAL
    user_name: str  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_linked_games(self, value: List[str]) -> ModelAccountProgressionInfo:
        self.linked_games = value
        return self

    def with_display_name(self, value: str) -> ModelAccountProgressionInfo:
        self.display_name = value
        return self

    def with_email(self, value: str) -> ModelAccountProgressionInfo:
        self.email = value
        return self

    def with_platform_id(self, value: str) -> ModelAccountProgressionInfo:
        self.platform_id = value
        return self

    def with_progressions(
        self, value: List[ModelProgressionInfo]
    ) -> ModelAccountProgressionInfo:
        self.progressions = value
        return self

    def with_unique_display_name(self, value: str) -> ModelAccountProgressionInfo:
        self.unique_display_name = value
        return self

    def with_user_name(self, value: str) -> ModelAccountProgressionInfo:
        self.user_name = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "linked_games"):
            result["linkedGames"] = [str(i0) for i0 in self.linked_games]
        elif include_empty:
            result["linkedGames"] = []
        if hasattr(self, "display_name"):
            result["displayName"] = str(self.display_name)
        elif include_empty:
            result["displayName"] = ""
        if hasattr(self, "email"):
            result["email"] = str(self.email)
        elif include_empty:
            result["email"] = ""
        if hasattr(self, "platform_id"):
            result["platformId"] = str(self.platform_id)
        elif include_empty:
            result["platformId"] = ""
        if hasattr(self, "progressions"):
            result["progressions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.progressions
            ]
        elif include_empty:
            result["progressions"] = []
        if hasattr(self, "unique_display_name"):
            result["uniqueDisplayName"] = str(self.unique_display_name)
        elif include_empty:
            result["uniqueDisplayName"] = ""
        if hasattr(self, "user_name"):
            result["userName"] = str(self.user_name)
        elif include_empty:
            result["userName"] = ""
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        linked_games: List[str],
        display_name: Optional[str] = None,
        email: Optional[str] = None,
        platform_id: Optional[str] = None,
        progressions: Optional[List[ModelProgressionInfo]] = None,
        unique_display_name: Optional[str] = None,
        user_name: Optional[str] = None,
        **kwargs,
    ) -> ModelAccountProgressionInfo:
        instance = cls()
        instance.linked_games = linked_games
        if display_name is not None:
            instance.display_name = display_name
        if email is not None:
            instance.email = email
        if platform_id is not None:
            instance.platform_id = platform_id
        if progressions is not None:
            instance.progressions = progressions
        if unique_display_name is not None:
            instance.unique_display_name = unique_display_name
        if user_name is not None:
            instance.user_name = user_name
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelAccountProgressionInfo:
        instance = cls()
        if not dict_:
            return instance
        if "linkedGames" in dict_ and dict_["linkedGames"] is not None:
            instance.linked_games = [str(i0) for i0 in dict_["linkedGames"]]
        elif include_empty:
            instance.linked_games = []
        if "displayName" in dict_ and dict_["displayName"] is not None:
            instance.display_name = str(dict_["displayName"])
        elif include_empty:
            instance.display_name = ""
        if "email" in dict_ and dict_["email"] is not None:
            instance.email = str(dict_["email"])
        elif include_empty:
            instance.email = ""
        if "platformId" in dict_ and dict_["platformId"] is not None:
            instance.platform_id = str(dict_["platformId"])
        elif include_empty:
            instance.platform_id = ""
        if "progressions" in dict_ and dict_["progressions"] is not None:
            instance.progressions = [
                ModelProgressionInfo.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["progressions"]
            ]
        elif include_empty:
            instance.progressions = []
        if "uniqueDisplayName" in dict_ and dict_["uniqueDisplayName"] is not None:
            instance.unique_display_name = str(dict_["uniqueDisplayName"])
        elif include_empty:
            instance.unique_display_name = ""
        if "userName" in dict_ and dict_["userName"] is not None:
            instance.user_name = str(dict_["userName"])
        elif include_empty:
            instance.user_name = ""
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelAccountProgressionInfo]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelAccountProgressionInfo]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelAccountProgressionInfo,
        List[ModelAccountProgressionInfo],
        Dict[Any, ModelAccountProgressionInfo],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "linkedGames": "linked_games",
            "displayName": "display_name",
            "email": "email",
            "platformId": "platform_id",
            "progressions": "progressions",
            "uniqueDisplayName": "unique_display_name",
            "userName": "user_name",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "linkedGames": True,
            "displayName": False,
            "email": False,
            "platformId": False,
            "progressions": False,
            "uniqueDisplayName": False,
            "userName": False,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the Custom Service Manager."""

__version__ = "1.34.0"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .create_sql_cluster_v2 import CreateSQLClusterV2
from .create_sql_database_cre_9da946 import CreateSQLDatabaseCredentialV2
from .create_sql_database_v2 import CreateSQLDatabaseV2
from .delete_sql_cluster_v2 import DeleteSQLClusterV2
from .delete_sql_database_v2 import DeleteSQLDatabaseV2
from .get_sql_app_list_v2 import GetSQLAppListV2
from .get_sql_cluster_v2 import GetSQLClusterV2
from .get_sql_database_v2 import GetSQLDatabaseV2
from .start_sql_cluster_v2 import StartSQLClusterV2
from .stop_sql_cluster_v2 import StopSQLClusterV2
from .update_sql_cluster_v2 import UpdateSQLClusterV2



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

"""Auto-generated package that contains models used by the AccelByte Gaming Services Iam Service."""

__version__ = "7.40.1"
__author__ = "AccelByte"
__email__ = "dev@accelbyte.net"

# pylint: disable=line-too-long

from .admin_retrieve_user_thi_205ff1 import AdminRetrieveUserThirdPartyPlatformTokenV3
from .auth_code_request_v3 import AuthCodeRequestV3
from .authorize_v3 import AuthorizeV3
from .authorize_v3 import (
    CodeChallengeMethodEnum as AuthorizeV3CodeChallengeMethodEnum,
    ResponseTypeEnum as AuthorizeV3ResponseTypeEnum,
)
from .change2fa_method import Change2faMethod
from .get_authorization_serve_d0fc55 import GetAuthorizationServerMetadataWithNamespace
from .get_jwksv3 import GetJWKSV3
from .get_revocation_list_v3 import GetRevocationListV3
from .o_auth_dynamic_client_r_de5475 import OAuthDynamicClientRegisterWithNamespaceV3
from .platform_token_grant_v3 import PlatformTokenGrantV3
from .retrieve_user_third_par_204b6e import RetrieveUserThirdPartyPlatformTokenV3
from .revoke_user_v3 import RevokeUserV3
from .send_mfa_authentication_code import SendMFAAuthenticationCode
from .simultaneous_login_v3 import SimultaneousLoginV3
from .simultaneous_login_v3 import (
    NativePlatformEnum as SimultaneousLoginV3NativePlatformEnum,
)
from .token_grant_v3 import TokenGrantV3
from .token_grant_v3 import GrantTypeEnum as TokenGrantV3GrantTypeEnum
from .token_introspection_v3 import TokenIntrospectionV3
from .token_revocation_v3 import TokenRevocationV3
from .verify2fa_code import Verify2faCode
from .verify2fa_code_forward import Verify2faCodeForward
from .verify_token_v3 import VerifyTokenV3



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ApiBackFillAcceptRequest
from ..models import ApiBackFillCreateRequest
from ..models import ApiBackFillRejectRequest
from ..models import ApiBackfillCreateResponse
from ..models import ApiBackfillGetResponse
from ..models import ApiBackfillProposalResponse
from ..models import ApiListBackfillQueryResponse
from ..models import ModelsGameSession
from ..models import ResponseError

from ..operations.backfill import AcceptBackfill
from ..operations.backfill import AdminQueryBackfill
from ..operations.backfill import CreateBackfill
from ..operations.backfill import DeleteBackfill
from ..operations.backfill import GetBackfill
from ..operations.backfill import GetBackfillProposal
from ..operations.backfill import RejectBackfill


@same_doc_as(AcceptBackfill)
def accept_backfill(
    backfill_id: str,
    body: ApiBackFillAcceptRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Accept a backfill proposal (AcceptBackfill)

    Accept backfill proposal.
    Field **acceptedTicketIds** can be used to accept specific tickets within a backfill proposal. If the ticketIDs are not mentioned in this field, those tickets will be rejected and reactivated for future proposals.
    If **acceptedTicketIds** is nil or not specified, then all tickets in the proposal will be accepted.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/accept

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillAcceptRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameSession (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AcceptBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AcceptBackfill)
async def accept_backfill_async(
    backfill_id: str,
    body: ApiBackFillAcceptRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Accept a backfill proposal (AcceptBackfill)

    Accept backfill proposal.
    Field **acceptedTicketIds** can be used to accept specific tickets within a backfill proposal. If the ticketIDs are not mentioned in this field, those tickets will be rejected and reactivated for future proposals.
    If **acceptedTicketIds** is nil or not specified, then all tickets in the proposal will be accepted.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/accept

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillAcceptRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ModelsGameSession (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AcceptBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(AdminQueryBackfill)
def admin_query_backfill(
    from_time: Optional[str] = None,
    is_active: Optional[bool] = None,
    limit: Optional[int] = None,
    match_pool: Optional[str] = None,
    offset: Optional[int] = None,
    player_id: Optional[str] = None,
    region: Optional[str] = None,
    session_id: Optional[str] = None,
    to_time: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin Query backfill ticket  (AdminQueryBackfill)

    Admin Query backfill ticket.

    Supports filtering by session ID, match pool, time range, active status, player ID, and region.
    Use pagination parameters offset and limit to control result size.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: GET

        tags: ["Backfill", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        from_time: (fromTime) OPTIONAL str in query

        is_active: (isActive) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        match_pool: (matchPool) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        player_id: (playerID) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        session_id: (sessionID) OPTIONAL str in query

        to_time: (toTime) OPTIONAL str in query

    Responses:
        200: OK - ApiListBackfillQueryResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryBackfill.create(
        from_time=from_time,
        is_active=is_active,
        limit=limit,
        match_pool=match_pool,
        offset=offset,
        player_id=player_id,
        region=region,
        session_id=session_id,
        to_time=to_time,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(AdminQueryBackfill)
async def admin_query_backfill_async(
    from_time: Optional[str] = None,
    is_active: Optional[bool] = None,
    limit: Optional[int] = None,
    match_pool: Optional[str] = None,
    offset: Optional[int] = None,
    player_id: Optional[str] = None,
    region: Optional[str] = None,
    session_id: Optional[str] = None,
    to_time: Optional[str] = None,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Admin Query backfill ticket  (AdminQueryBackfill)

    Admin Query backfill ticket.

    Supports filtering by session ID, match pool, time range, active status, player ID, and region.
    Use pagination parameters offset and limit to control result size.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: GET

        tags: ["Backfill", "admin"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        from_time: (fromTime) OPTIONAL str in query

        is_active: (isActive) OPTIONAL bool in query

        limit: (limit) OPTIONAL int in query

        match_pool: (matchPool) OPTIONAL str in query

        offset: (offset) OPTIONAL int in query

        player_id: (playerID) OPTIONAL str in query

        region: (region) OPTIONAL str in query

        session_id: (sessionID) OPTIONAL str in query

        to_time: (toTime) OPTIONAL str in query

    Responses:
        200: OK - ApiListBackfillQueryResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = AdminQueryBackfill.create(
        from_time=from_time,
        is_active=is_active,
        limit=limit,
        match_pool=match_pool,
        offset=offset,
        player_id=player_id,
        region=region,
        session_id=session_id,
        to_time=to_time,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(CreateBackfill)
def create_backfill(
    body: ApiBackFillCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a backfill ticket (CreateBackfill)

    Create backfill ticket.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: POST

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApiBackfillCreateResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateBackfill.create(
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(CreateBackfill)
async def create_backfill_async(
    body: ApiBackFillCreateRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Create a backfill ticket (CreateBackfill)

    Create backfill ticket.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill

        method: POST

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillCreateRequest in body

        namespace: (namespace) REQUIRED str in path

    Responses:
        201: Created - ApiBackfillCreateResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = CreateBackfill.create(
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(DeleteBackfill)
def delete_backfill(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a backfill ticket (DeleteBackfill)

    Delete backfill ticket.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: DELETE

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(DeleteBackfill)
async def delete_backfill_async(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Delete a backfill ticket (DeleteBackfill)

    Delete backfill ticket.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: DELETE

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        204: No Content - (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = DeleteBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetBackfill)
def get_backfill(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a backfill ticket (GetBackfill)

    Get backfill ticket by ID

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiBackfillGetResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetBackfill)
async def get_backfill_async(
    backfill_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get a backfill ticket (GetBackfill)

    Get backfill ticket by ID

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - ApiBackfillGetResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetBackfill.create(
        backfill_id=backfill_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(GetBackfillProposal)
def get_backfill_proposal(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get backfill proposal (GetBackfillProposal)

    Get backfill proposal for the given session.

    Returns the current pending backfill proposal associated with the session ID.
    Returns 404 if no proposal exists for the session.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/proposal

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in query

    Responses:
        200: OK - ApiBackfillProposalResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetBackfillProposal.create(
        session_id=session_id,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(GetBackfillProposal)
async def get_backfill_proposal_async(
    session_id: str,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Get backfill proposal (GetBackfillProposal)

    Get backfill proposal for the given session.

    Returns the current pending backfill proposal associated with the session ID.
    Returns 404 if no proposal exists for the session.

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/proposal

        method: GET

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        namespace: (namespace) REQUIRED str in path

        session_id: (sessionID) REQUIRED str in query

    Responses:
        200: OK - ApiBackfillProposalResponse (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = GetBackfillProposal.create(
        session_id=session_id,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(RejectBackfill)
def reject_backfill(
    backfill_id: str,
    body: ApiBackFillRejectRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reject a backfill proposal (RejectBackfill)

    Reject backfill proposal

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/reject

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillRejectRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RejectBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(RejectBackfill)
async def reject_backfill_async(
    backfill_id: str,
    body: ApiBackFillRejectRequest,
    namespace: Optional[str] = None,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Reject a backfill proposal (RejectBackfill)

    Reject backfill proposal

    Properties:
        url: /match2/v1/namespaces/{namespace}/backfill/{backfillID}/proposal/reject

        method: PUT

        tags: ["Backfill", "public"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        body: (body) REQUIRED ApiBackFillRejectRequest in body

        backfill_id: (backfillID) REQUIRED str in path

        namespace: (namespace) REQUIRED str in path

    Responses:
        200: OK - (OK)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    if namespace is None:
        namespace, error = get_services_namespace(sdk=kwargs.get("sdk"))
        if error:
            return None, error
    request = RejectBackfill.create(
        backfill_id=backfill_id,
        body=body,
        namespace=namespace,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# Custom Service Manager

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.domain_allowed_intercepted_port import DomainAllowedInterceptedPort
from ..models.domain_debug_pod import DomainDebugPod
from ..models.domain_exposed_service import DomainExposedService


class ApimodelAppDebugInfoResponse(Model):
    """Apimodel app debug info response (apimodel.AppDebugInfoResponse)

    Properties:
        allowed_intercepted_ports: (allowedInterceptedPorts) REQUIRED List[DomainAllowedInterceptedPort]

        app_status: (appStatus) REQUIRED str

        debug_pods: (debugPods) REQUIRED List[DomainDebugPod]

        exposed_services: (exposedServices) REQUIRED List[DomainExposedService]

        is_debug_mode_enabled: (isDebugModeEnabled) REQUIRED bool

        is_debug_session_connected: (isDebugSessionConnected) REQUIRED bool
    """

    # region fields

    allowed_intercepted_ports: List[DomainAllowedInterceptedPort]  # REQUIRED
    app_status: str  # REQUIRED
    debug_pods: List[DomainDebugPod]  # REQUIRED
    exposed_services: List[DomainExposedService]  # REQUIRED
    is_debug_mode_enabled: bool  # REQUIRED
    is_debug_session_connected: bool  # REQUIRED

    # endregion fields

    # region with_x methods

    def with_allowed_intercepted_ports(
        self, value: List[DomainAllowedInterceptedPort]
    ) -> ApimodelAppDebugInfoResponse:
        self.allowed_intercepted_ports = value
        return self

    def with_app_status(self, value: str) -> ApimodelAppDebugInfoResponse:
        self.app_status = value
        return self

    def with_debug_pods(
        self, value: List[DomainDebugPod]
    ) -> ApimodelAppDebugInfoResponse:
        self.debug_pods = value
        return self

    def with_exposed_services(
        self, value: List[DomainExposedService]
    ) -> ApimodelAppDebugInfoResponse:
        self.exposed_services = value
        return self

    def with_is_debug_mode_enabled(self, value: bool) -> ApimodelAppDebugInfoResponse:
        self.is_debug_mode_enabled = value
        return self

    def with_is_debug_session_connected(
        self, value: bool
    ) -> ApimodelAppDebugInfoResponse:
        self.is_debug_session_connected = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "allowed_intercepted_ports"):
            result["allowedInterceptedPorts"] = [
                i0.to_dict(include_empty=include_empty)
                for i0 in self.allowed_intercepted_ports
            ]
        elif include_empty:
            result["allowedInterceptedPorts"] = []
        if hasattr(self, "app_status"):
            result["appStatus"] = str(self.app_status)
        elif include_empty:
            result["appStatus"] = ""
        if hasattr(self, "debug_pods"):
            result["debugPods"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.debug_pods
            ]
        elif include_empty:
            result["debugPods"] = []
        if hasattr(self, "exposed_services"):
            result["exposedServices"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.exposed_services
            ]
        elif include_empty:
            result["exposedServices"] = []
        if hasattr(self, "is_debug_mode_enabled"):
            result["isDebugModeEnabled"] = bool(self.is_debug_mode_enabled)
        elif include_empty:
            result["isDebugModeEnabled"] = False
        if hasattr(self, "is_debug_session_connected"):
            result["isDebugSessionConnected"] = bool(self.is_debug_session_connected)
        elif include_empty:
            result["isDebugSessionConnected"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        allowed_intercepted_ports: List[DomainAllowedInterceptedPort],
        app_status: str,
        debug_pods: List[DomainDebugPod],
        exposed_services: List[DomainExposedService],
        is_debug_mode_enabled: bool,
        is_debug_session_connected: bool,
        **kwargs,
    ) -> ApimodelAppDebugInfoResponse:
        instance = cls()
        instance.allowed_intercepted_ports = allowed_intercepted_ports
        instance.app_status = app_status
        instance.debug_pods = debug_pods
        instance.exposed_services = exposed_services
        instance.is_debug_mode_enabled = is_debug_mode_enabled
        instance.is_debug_session_connected = is_debug_session_connected
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ApimodelAppDebugInfoResponse:
        instance = cls()
        if not dict_:
            return instance
        if (
            "allowedInterceptedPorts" in dict_
            and dict_["allowedInterceptedPorts"] is not None
        ):
            instance.allowed_intercepted_ports = [
                DomainAllowedInterceptedPort.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["allowedInterceptedPorts"]
            ]
        elif include_empty:
            instance.allowed_intercepted_ports = []
        if "appStatus" in dict_ and dict_["appStatus"] is not None:
            instance.app_status = str(dict_["appStatus"])
        elif include_empty:
            instance.app_status = ""
        if "debugPods" in dict_ and dict_["debugPods"] is not None:
            instance.debug_pods = [
                DomainDebugPod.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["debugPods"]
            ]
        elif include_empty:
            instance.debug_pods = []
        if "exposedServices" in dict_ and dict_["exposedServices"] is not None:
            instance.exposed_services = [
                DomainExposedService.create_from_dict(i0, include_empty=include_empty)
                for i0 in dict_["exposedServices"]
            ]
        elif include_empty:
            instance.exposed_services = []
        if "isDebugModeEnabled" in dict_ and dict_["isDebugModeEnabled"] is not None:
            instance.is_debug_mode_enabled = bool(dict_["isDebugModeEnabled"])
        elif include_empty:
            instance.is_debug_mode_enabled = False
        if (
            "isDebugSessionConnected" in dict_
            and dict_["isDebugSessionConnected"] is not None
        ):
            instance.is_debug_session_connected = bool(dict_["isDebugSessionConnected"])
        elif include_empty:
            instance.is_debug_session_connected = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ApimodelAppDebugInfoResponse]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ApimodelAppDebugInfoResponse]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ApimodelAppDebugInfoResponse,
        List[ApimodelAppDebugInfoResponse],
        Dict[Any, ApimodelAppDebugInfoResponse],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "allowedInterceptedPorts": "allowed_intercepted_ports",
            "appStatus": "app_status",
            "debugPods": "debug_pods",
            "exposedServices": "exposed_services",
            "isDebugModeEnabled": "is_debug_mode_enabled",
            "isDebugSessionConnected": "is_debug_session_connected",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "allowedInterceptedPorts": True,
            "appStatus": True,
            "debugPods": True,
            "exposedServices": True,
            "isDebugModeEnabled": True,
            "isDebugSessionConnected": True,
        }

    # endregion static methods



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import HeaderStr
from ....core import get_namespace as get_services_namespace
from ....core import run_request
from ....core import run_request_async
from ....core import same_doc_as

from ..models import ModelsDeletionStatus
from ..models import ModelsRequestDeleteResponse
from ..models import ResponseError

from ..operations.data_deletion_headless import PublicCancelMyAccountDeletionRequest
from ..operations.data_deletion_headless import PublicGetMyAccountDeletionStatus
from ..operations.data_deletion_headless import PublicSubmitMyAccountDeletionRequest
from ..operations.data_deletion_headless import PublicSubmitMyHeadlessDeletionRequest


@same_doc_as(PublicCancelMyAccountDeletionRequest)
def public_cancel_my_account_deletion_request(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Cancel my account deletion request (PublicCancelMyAccountDeletionRequest)

    Cancel my account deletion request
    Requires valid user access token

    Properties:
        url: /gdpr/public/users/me/deletions

        method: DELETE

        tags: ["Data Deletion - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicCancelMyAccountDeletionRequest.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicCancelMyAccountDeletionRequest)
async def public_cancel_my_account_deletion_request_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Cancel my account deletion request (PublicCancelMyAccountDeletionRequest)

    Cancel my account deletion request
    Requires valid user access token

    Properties:
        url: /gdpr/public/users/me/deletions

        method: DELETE

        tags: ["Data Deletion - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        204: No Content - (No Content)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicCancelMyAccountDeletionRequest.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicGetMyAccountDeletionStatus)
def public_get_my_account_deletion_status(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve my account deletion status (PublicGetMyAccountDeletionStatus)

    Retrieve my account deletion status
    Requires valid user access token

    Properties:
        url: /gdpr/public/users/me/deletions/status

        method: GET

        tags: ["Data Deletion - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ModelsDeletionStatus (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGetMyAccountDeletionStatus.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicGetMyAccountDeletionStatus)
async def public_get_my_account_deletion_status_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Retrieve my account deletion status (PublicGetMyAccountDeletionStatus)

    Retrieve my account deletion status
    Requires valid user access token

    Properties:
        url: /gdpr/public/users/me/deletions/status

        method: GET

        tags: ["Data Deletion - Headless"]

        consumes: ["application/json"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        200: OK - ModelsDeletionStatus (OK)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicGetMyAccountDeletionStatus.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicSubmitMyAccountDeletionRequest)
def public_submit_my_account_deletion_request(
    platform_id: str,
    platform_token: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my account deletion request (PublicSubmitMyAccountDeletionRequest)

    Submit an Account Deletion Request authenticated by a platform token.
    Requires a valid access token.

    This endpoint is intended for **in-game use** only and requires a valid platformId and platform token.
    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/deletions` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/deletions

        method: POST

        tags: ["Data Deletion - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in form_data

        platform_token: (platformToken) REQUIRED str in form_data

    Responses:
        201: Created - ModelsRequestDeleteResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyAccountDeletionRequest.create(
        platform_id=platform_id,
        platform_token=platform_token,
    )
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicSubmitMyAccountDeletionRequest)
async def public_submit_my_account_deletion_request_async(
    platform_id: str,
    platform_token: str,
    x_additional_headers: Optional[Dict[str, str]] = None,
    **kwargs
):
    """Submit my account deletion request (PublicSubmitMyAccountDeletionRequest)

    Submit an Account Deletion Request authenticated by a platform token.
    Requires a valid access token.

    This endpoint is intended for **in-game use** only and requires a valid platformId and platform token.
    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/deletions` instead.

    ### Request Header:
    - **Content-Type: application/x-www-form-urlencoded**

    Properties:
        url: /gdpr/public/users/me/deletions

        method: POST

        tags: ["Data Deletion - Headless"]

        consumes: ["application/x-www-form-urlencoded"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

        platform_id: (platformId) REQUIRED str in form_data

        platform_token: (platformToken) REQUIRED str in form_data

    Responses:
        201: Created - ModelsRequestDeleteResponse (Created)

        400: Bad Request - ResponseError (Bad Request)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        404: Not Found - ResponseError (Not Found)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyAccountDeletionRequest.create(
        platform_id=platform_id,
        platform_token=platform_token,
    )
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )


@same_doc_as(PublicSubmitMyHeadlessDeletionRequest)
def public_submit_my_headless_deletion_request(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Submit my headless account deletion request (PublicSubmitMyHeadlessDeletionRequest)

    Submit an Account Deletion Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 10 seconds).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/deletions` instead.

    Properties:
        url: /gdpr/public/users/me/headless/deletions

        method: POST

        tags: ["Data Deletion - Headless"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        201: Created - ModelsRequestDeleteResponse (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyHeadlessDeletionRequest.create()
    return run_request(request, additional_headers=x_additional_headers, **kwargs)


@same_doc_as(PublicSubmitMyHeadlessDeletionRequest)
async def public_submit_my_headless_deletion_request_async(
    x_additional_headers: Optional[Dict[str, str]] = None, **kwargs
):
    """Submit my headless account deletion request (PublicSubmitMyHeadlessDeletionRequest)

    Submit an Account Deletion Request for headless account scenario (without a password).
    **Typically used by web portal** to invoke headless player GDPR using 3rd platform web authentication.
    Requires a valid access token and a fresh **gdpr_token** cookie (max-age 10 seconds).
    The gdpr_token is issued by the IAM service during 3rd platform web authentication.

    If a full account is available, use `POST /gdpr/public/namespaces/{namespace}/users/{userId}/deletions` instead.

    Properties:
        url: /gdpr/public/users/me/headless/deletions

        method: POST

        tags: ["Data Deletion - Headless"]

        consumes: ["*/*"]

        produces: ["application/json"]

        securities: [BEARER_AUTH]

    Responses:
        201: Created - ModelsRequestDeleteResponse (Created)

        401: Unauthorized - ResponseError (Unauthorized)

        403: Forbidden - ResponseError (Forbidden)

        409: Conflict - ResponseError (Conflict)

        500: Internal Server Error - ResponseError (Internal Server Error)
    """
    request = PublicSubmitMyHeadlessDeletionRequest.create()
    return await run_request_async(
        request, additional_headers=x_additional_headers, **kwargs
    )



# Copyright (c) 2021 AccelByte Inc. All Rights Reserved.
# This is licensed software from AccelByte Inc, for limitations
# and restrictions contact your company contract manager.
#
# Code generated. DO NOT EDIT!

# template file: ags_py_codegen

# AccelByte Gaming Services Iam Service

# pylint: disable=duplicate-code
# pylint: disable=line-too-long
# pylint: disable=missing-function-docstring
# pylint: disable=missing-module-docstring
# pylint: disable=too-many-arguments
# pylint: disable=too-many-branches
# pylint: disable=too-many-instance-attributes
# pylint: disable=too-many-lines
# pylint: disable=too-many-locals
# pylint: disable=too-many-public-methods
# pylint: disable=too-many-return-statements
# pylint: disable=too-many-statements
# pylint: disable=unused-import

from __future__ import annotations
from typing import Any, Dict, List, Optional, Tuple, Union

from ....core import Model

from ..models.accountcommon_permission_v3 import AccountcommonPermissionV3
from ..models.accountcommon_role_manager_v3 import AccountcommonRoleManagerV3
from ..models.accountcommon_role_member_v3 import AccountcommonRoleMemberV3


class ModelRoleCreateV3Request(Model):
    """Model role create V3 request (model.RoleCreateV3Request)

    Properties:
        admin_role: (adminRole) REQUIRED bool

        is_wildcard: (isWildcard) REQUIRED bool

        managers: (managers) REQUIRED List[AccountcommonRoleManagerV3]

        members: (members) REQUIRED List[AccountcommonRoleMemberV3]

        permissions: (permissions) REQUIRED List[AccountcommonPermissionV3]

        role_name: (roleName) REQUIRED str

        deletable: (deletable) OPTIONAL bool
    """

    # region fields

    admin_role: bool  # REQUIRED
    is_wildcard: bool  # REQUIRED
    managers: List[AccountcommonRoleManagerV3]  # REQUIRED
    members: List[AccountcommonRoleMemberV3]  # REQUIRED
    permissions: List[AccountcommonPermissionV3]  # REQUIRED
    role_name: str  # REQUIRED
    deletable: bool  # OPTIONAL

    # endregion fields

    # region with_x methods

    def with_admin_role(self, value: bool) -> ModelRoleCreateV3Request:
        self.admin_role = value
        return self

    def with_is_wildcard(self, value: bool) -> ModelRoleCreateV3Request:
        self.is_wildcard = value
        return self

    def with_managers(
        self, value: List[AccountcommonRoleManagerV3]
    ) -> ModelRoleCreateV3Request:
        self.managers = value
        return self

    def with_members(
        self, value: List[AccountcommonRoleMemberV3]
    ) -> ModelRoleCreateV3Request:
        self.members = value
        return self

    def with_permissions(
        self, value: List[AccountcommonPermissionV3]
    ) -> ModelRoleCreateV3Request:
        self.permissions = value
        return self

    def with_role_name(self, value: str) -> ModelRoleCreateV3Request:
        self.role_name = value
        return self

    def with_deletable(self, value: bool) -> ModelRoleCreateV3Request:
        self.deletable = value
        return self

    # endregion with_x methods

    # region to methods

    def to_dict(self, include_empty: bool = False) -> dict:
        result: dict = {}
        if hasattr(self, "admin_role"):
            result["adminRole"] = bool(self.admin_role)
        elif include_empty:
            result["adminRole"] = False
        if hasattr(self, "is_wildcard"):
            result["isWildcard"] = bool(self.is_wildcard)
        elif include_empty:
            result["isWildcard"] = False
        if hasattr(self, "managers"):
            result["managers"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.managers
            ]
        elif include_empty:
            result["managers"] = []
        if hasattr(self, "members"):
            result["members"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.members
            ]
        elif include_empty:
            result["members"] = []
        if hasattr(self, "permissions"):
            result["permissions"] = [
                i0.to_dict(include_empty=include_empty) for i0 in self.permissions
            ]
        elif include_empty:
            result["permissions"] = []
        if hasattr(self, "role_name"):
            result["roleName"] = str(self.role_name)
        elif include_empty:
            result["roleName"] = ""
        if hasattr(self, "deletable"):
            result["deletable"] = bool(self.deletable)
        elif include_empty:
            result["deletable"] = False
        return result

    # endregion to methods

    # region static methods

    @classmethod
    def create(
        cls,
        admin_role: bool,
        is_wildcard: bool,
        managers: List[AccountcommonRoleManagerV3],
        members: List[AccountcommonRoleMemberV3],
        permissions: List[AccountcommonPermissionV3],
        role_name: str,
        deletable: Optional[bool] = None,
        **kwargs,
    ) -> ModelRoleCreateV3Request:
        instance = cls()
        instance.admin_role = admin_role
        instance.is_wildcard = is_wildcard
        instance.managers = managers
        instance.members = members
        instance.permissions = permissions
        instance.role_name = role_name
        if deletable is not None:
            instance.deletable = deletable
        return instance

    @classmethod
    def create_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> ModelRoleCreateV3Request:
        instance = cls()
        if not dict_:
            return instance
        if "adminRole" in dict_ and dict_["adminRole"] is not None:
            instance.admin_role = bool(dict_["adminRole"])
        elif include_empty:
            instance.admin_role = False
        if "isWildcard" in dict_ and dict_["isWildcard"] is not None:
            instance.is_wildcard = bool(dict_["isWildcard"])
        elif include_empty:
            instance.is_wildcard = False
        if "managers" in dict_ and dict_["managers"] is not None:
            instance.managers = [
                AccountcommonRoleManagerV3.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["managers"]
            ]
        elif include_empty:
            instance.managers = []
        if "members" in dict_ and dict_["members"] is not None:
            instance.members = [
                AccountcommonRoleMemberV3.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["members"]
            ]
        elif include_empty:
            instance.members = []
        if "permissions" in dict_ and dict_["permissions"] is not None:
            instance.permissions = [
                AccountcommonPermissionV3.create_from_dict(
                    i0, include_empty=include_empty
                )
                for i0 in dict_["permissions"]
            ]
        elif include_empty:
            instance.permissions = []
        if "roleName" in dict_ and dict_["roleName"] is not None:
            instance.role_name = str(dict_["roleName"])
        elif include_empty:
            instance.role_name = ""
        if "deletable" in dict_ and dict_["deletable"] is not None:
            instance.deletable = bool(dict_["deletable"])
        elif include_empty:
            instance.deletable = False
        return instance

    @classmethod
    def create_many_from_dict(
        cls, dict_: dict, include_empty: bool = False
    ) -> Dict[str, ModelRoleCreateV3Request]:
        return (
            {k: cls.create_from_dict(v, include_empty=include_empty) for k, v in dict_}
            if dict_
            else {}
        )

    @classmethod
    def create_many_from_list(
        cls, list_: list, include_empty: bool = False
    ) -> List[ModelRoleCreateV3Request]:
        return (
            [cls.create_from_dict(i, include_empty=include_empty) for i in list_]
            if list_
            else []
        )

    @classmethod
    def create_from_any(
        cls, any_: any, include_empty: bool = False, many: bool = False
    ) -> Union[
        ModelRoleCreateV3Request,
        List[ModelRoleCreateV3Request],
        Dict[Any, ModelRoleCreateV3Request],
    ]:
        if many:
            if isinstance(any_, dict):
                return cls.create_many_from_dict(any_, include_empty=include_empty)
            elif isinstance(any_, list):
                return cls.create_many_from_list(any_, include_empty=include_empty)
            else:
                raise ValueError()
        else:
            return cls.create_from_dict(any_, include_empty=include_empty)

    @staticmethod
    def get_field_info() -> Dict[str, str]:
        return {
            "adminRole": "admin_role",
            "isWildcard": "is_wildcard",
            "managers": "managers",
            "members": "members",
            "permissions": "permissions",
            "roleName": "role_name",
            "deletable": "deletable",
        }

    @staticmethod
    def get_required_map() -> Dict[str, bool]:
        return {
            "adminRole": True,
            "isWildcard": True,
            "managers": True,
            "members": True,
            "permissions": True,
            "roleName": True,
            "deletable": False,
        }

    # endregion static methods
